#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

import gc


# PauseGC is a class for clean, scoped management of the Python
# garbage collector.  To disable the GC for the duration of a
# scoped block use PauseGC in combination with the Python 'with'
# statement. E.g.,
#
# def my_func():
#    with PauseGC() as pgc:
#       [...]
#
# When the instance falls out of scope (by termination or exception),
# the GC will be re-enabled (if it was not initially disabled).  It is
# safe to nest instances of PauseGC That is, you don't have to worry
# if an outer function/method has its own instance of PauseGC.
class PauseGC(object):

    __slots__ = ("reenable_gc",)

    def __init__(self, pause=True):
        self.reenable_gc = False
        if pause:
            try:
                self.reenable_gc = gc.isenabled()
                gc.disable()
            except NotImplementedError:
                # This will only happen if the Python implementation
                # doesn't support disabling the GC.
                pass

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()

    def close(self):
        if self.reenable_gc:
            gc.enable()
        self.reenable_gc = False



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________
"""A plugin that manages temporary files."""

__all__ = ['ITempfileManager', 'TempfileManagerPlugin', 'TempfileManager']

import os
import time
import tempfile
import logging
import shutil

from pyutilib.component.core import Interface, implements
from pyutilib.component.config.managed_plugin import ManagedSingletonPlugin
from pyutilib.component.config.options import declare_option

deletion_errors_are_fatal = True


class ITempfileManager(Interface):
    """Interface for managing temporary files."""

    def create_tempfile(self, suffix=None, prefix=None, text=False, dir=None):
        """Return the absolute path of a temporary file that is guaranteed to be unique."""

    def create_tempdir(self, suffix=None, prefix=None, dir=None):
        """Return the absolute path of a temporary directory that is guaranteed to be unique."""

    def add_tempfile(self, filename):
        """Declare this file to be temporary."""

    def clear_tempfiles(self, remove=True):
        """Delete all temporary files."""

    def sequential_files(self, ctr=0):
        """Start generating sequential files, using the specified counter"""

    def unique_files(self):
        """Start generating unique files"""

    def push(self):
        """Push tempfiles onto a stack.  This creates a new context for tempfiles."""

    def pop(self, remove=True):
        """Pop tempfiles onto a stack.  Tempfiles that are popped off the stack are deleted."""


class TempfileManagerPlugin(ManagedSingletonPlugin):
    """A plugin that manages temporary files."""

    implements(ITempfileManager)

    def __init__(self, **kwds):
        if getattr(self, '_initialized', False) is True:
            return
        self._initialized = True
        #
        kwds['name'] = 'TempfileManager'
        ManagedSingletonPlugin.__init__(self, **kwds)
        self._tempfiles = [[]]
        declare_option("tempdir", default=None)
        self._ctr = -1

    def create_tempfile(self, suffix=None, prefix=None, text=False, dir=None):
        """
        Return the absolute path of a temporary filename that is
        guaranteed to be unique.  This function generates the file and returns
        the filename.
        """
        if suffix is None:
            suffix = ''
        if prefix is None:
            prefix = 'tmp'
        if dir is None:
            dir = self.tempdir

        ans = tempfile.mkstemp(suffix=suffix, prefix=prefix, text=text, dir=dir)
        ans = list(ans)
        if not os.path.isabs(ans[1]):  #pragma:nocover
            fname = os.path.join(dir, ans[1])
        else:
            fname = ans[1]
        os.close(ans[0])
        if self._ctr >= 0:
            new_fname = os.path.join(dir, prefix + str(self._ctr) + suffix)
            # Delete any file having the sequential name and then
            # rename
            if os.path.exists(new_fname):
                os.remove(new_fname)
            shutil.move(fname, new_fname)
            fname = new_fname
            self._ctr += 1
        self._tempfiles[-1].append(fname)
        return fname

    def create_tempdir(self, suffix=None, prefix=None, dir=None):
        """
        Return the absolute path of a temporary directory that is
        guaranteed to be unique.  This function generates the directory and returns
        the directory name.
        """
        if suffix is None:
            suffix = ''
        if prefix is None:
            prefix = 'tmp'
        if dir is None:
            dir = self.tempdir
        dirname = tempfile.mkdtemp(suffix=suffix, prefix=prefix, dir=dir)
        if self._ctr >= 0:
            new_dirname = os.path.join(dir, prefix + str(self._ctr) + suffix)
            # Delete any directory having the sequential name and then
            # rename
            if os.path.exists(new_dirname):
                shutil.rmtree(new_dirname)
            shutil.move(dirname, new_dirname)
            dirname = new_dirname
            self._ctr += 1

        self._tempfiles[-1].append(dirname)
        return dirname

    def add_tempfile(self, filename, exists=True):
        """Declare this file to be temporary."""
        tmp = os.path.abspath(filename)
        if exists and not os.path.exists(tmp):
            raise IOError("Temporary file does not exist: " + tmp)
        self._tempfiles[-1].append(tmp)

    def clear_tempfiles(self, remove=True):
        """Delete all temporary files."""
        while len(self._tempfiles) > 1:
            self.pop(remove)
        self.pop(remove)

    def sequential_files(self, ctr=0):
        """Start generating sequential files, using the specified counter"""
        self._ctr = ctr

    def unique_files(self):
        """Stop generating sequential files, using the specified counter"""
        self._ctr = -1

    #
    # Support "with" statements, where the pop automatically
    # takes place on exit.
    #
    def push(self):
        self._tempfiles.append([])
        return self

    def __exit__(self, type, value, traceback):
        self.pop(remove=True)

    def pop(self, remove=True):
        files = self._tempfiles.pop()
        if remove:
            for filename in files:
                if os.path.exists(filename):
                    if os.path.isdir(filename):
                        shutil.rmtree(
                            filename,
                            ignore_errors=not deletion_errors_are_fatal)
                    else:
                        try:
                            os.remove(filename)
                        except WindowsError:
                            # Sometimes Windows doesn't release the
                            # file lock immediately when the process
                            # terminates.  If we get an error, wait a
                            # second and try again.
                            try:
                                time.sleep(1)
                                os.remove(filename)
                            except WindowsError:
                                if deletion_errors_are_fatal:
                                    raise
                                else:
                                    # Failure to delete a tempfile
                                    # should NOT be fatal
                                    logger = logging.getLogger(
                                        'pyutilib.component.config')
                                    logger.warning("Unable to delete temporary "
                                                   "file %s" % (filename,))

        if len(self._tempfiles) == 0:
            self._tempfiles = [[]]

#
# This global class provides a convenient handle to this
# singleton plugin
#
TempfileManager = TempfileManagerPlugin()



#
# Unit Tests for util/misc/import_file
#
#

import os
import sys
from os.path import abspath, dirname
pkgdir = dirname(abspath(__file__)) + os.sep + ".." + os.sep + ".."
currdir = dirname(abspath(__file__)) + os.sep

import pyutilib.misc
import pyutilib.th as unittest
import pyutilib.misc.comparison

try:
    import runpy
    _runpy = True
except:
    _runpy = False


class TestRunFile(unittest.TestCase):

    def test_run_file1(self):
        pyutilib.misc.run_file(
            currdir + "import1.py", logfile=currdir + "import1.log")
        if not os.path.exists(currdir + "import1.log"):
            self.fail("test_run_file - failed to create logfile")
        self.assertFalse(
            pyutilib.misc.comparison.compare_file(currdir + "import1.log",
                                                  currdir + "import1.txt")[0])
        os.remove(currdir + "import1.log")

    def test_run_file2(self):
        pyutilib.misc.run_file(
            "import1.py", logfile=currdir + "import1.log", execdir=currdir)
        if not os.path.exists(currdir + "import1.log"):
            self.fail("test_run_file - failed to create logfile")
        self.assertFalse(
            pyutilib.misc.comparison.compare_file(currdir + "import1.log",
                                                  currdir + "import1.txt")[0])
        os.remove(currdir + "import1.log")

    def test_run_file3(self):
        try:
            pyutilib.misc.run_file(
                "import2.py", logfile=currdir + "import2.log", execdir=currdir)
            self.fail("test_run_file - expected type error in import2.py")
        except TypeError:
            pass
        self.assertFalse(
            pyutilib.misc.comparison.compare_file(currdir + "import2.log",
                                                  currdir + "import2.txt")[0])
        os.remove(currdir + "import2.log")


class TestImportFile(unittest.TestCase):

    def setUp(self):
        self._mods = list(sys.modules.keys())

    def tearDown(self):
        to_del = [m for m in sys.modules.keys() if m not in self._mods]
        for mod in to_del:
            del sys.modules[mod]

    def test_import_file_context1(self):
        pyutilib.misc.import_file(currdir + "import1.py")
        if "import1" in globals():
            self.fail(
                "test_import_file - globals() should not be affected by import")

    def test_import_file_context2(self):
        import1 = pyutilib.misc.import_file(currdir + "import1.py")
        try:
            c = import1.a
        except:
            self.fail("test_import_file - could not access data in import.py")

    def test_import_file_context3(self):
        pyutilib.misc.import_file(currdir + "import1.py", context=globals())
        if not "import1" in globals():
            self.fail("test_import_file - failed to import the import1.py file")

    def test1(self):
        try:
            pyutilib.misc.import_file('tfile.py')
        except IOError:
            pass
        else:
            self.fail('File does not exist. Expected IOError.')

    def test2(self):
        try:
            pyutilib.misc.import_file('tfile')
        except ImportError:
            pass
        else:
            self.fail('Module does not exist. Expected ImportError.')

    def test3(self):
        dirname = currdir + 'import_data' + os.sep + 'a'
        sys.path.insert(0, dirname)
        context = {}
        m = pyutilib.misc.import_file('tfile', context=context)
        self.assertEqual(id(m), id(context['tfile']))
        self.assertEqual(m.f(), 'a')
        sys.path.remove(dirname)

    def test4(self):
        dirname = currdir + 'import_data' + os.sep + 'a'
        sys.path.insert(0, dirname)
        context = {}
        m = pyutilib.misc.import_file('tfile', context=context)
        self.assertEqual(id(m), id(context['tfile']))
        self.assertEqual(m.f(), 'a')
        sys.path.remove(dirname)
        dirname = currdir + 'import_data' + os.sep + 'b'
        sys.path.insert(0, dirname)
        m = pyutilib.misc.import_file('tfile', context=context, name='junk')
        self.assertEqual(id(m), id(context['junk']))
        self.assertEqual(m.f(), 'b')
        sys.path.remove(dirname)

    def test4a(self):
        dirname = currdir + 'import_data' + os.sep + 'a'
        sys.path.insert(0, dirname)
        context = {}
        m = pyutilib.misc.import_file('tfile', context=context)
        self.assertEqual(id(m), id(context['tfile']))
        self.assertEqual(m.f(), 'a')
        sys.path.remove(dirname)
        dirname = currdir + 'import_data' + os.sep + 'b'
        sys.path.insert(0, dirname)
        m = pyutilib.misc.import_file('tfile', context=context)
        self.assertEqual(id(m), id(context['tfile']))
        self.assertEqual(m.f(), 'a')
        sys.path.remove(dirname)

    def test5(self):
        dirname = currdir + 'import_data' + os.sep + 'a' + os.sep
        context = {}
        m = pyutilib.misc.import_file(dirname + 'tfile', context=context)
        self.assertEqual(id(m), id(context['tfile']))
        self.assertEqual(m.f(), 'a')

    def test6(self):
        dirname = currdir + 'import_data' + os.sep + 'a' + os.sep
        context = {}
        m = pyutilib.misc.import_file(dirname + 'tfile', context=context)
        self.assertEqual(id(m), id(context['tfile']))
        self.assertEqual(m.f(), 'a')
        dirname = currdir + 'import_data' + os.sep + 'b' + os.sep
        m = pyutilib.misc.import_file(
            dirname + 'tfile', context=context, name='junk')
        self.assertEqual(id(m), id(context['junk']))
        self.assertEqual(m.f(), 'b')

    def test6a(self):
        dirname = currdir + 'import_data' + os.sep + 'a' + os.sep
        context = {}
        m = pyutilib.misc.import_file(dirname + 'tfile', context=context)
        self.assertEqual(id(m), id(context['tfile']))
        self.assertEqual(m.f(), 'a')
        dirname = currdir + 'import_data' + os.sep + 'b' + os.sep
        m = pyutilib.misc.import_file(dirname + 'tfile', context=context)
        self.assertEqual(id(m), id(context['tfile']))
        self.assertEqual(m.f(), 'a')

    def test7(self):
        dirname = currdir + 'import_data' + os.sep + 'a' + os.sep
        context = {}
        m = pyutilib.misc.import_file(dirname + 'tfile.py', context=context)
        self.assertEqual(id(m), id(context['tfile']))
        self.assertEqual(m.f(), 'a')

    def test8(self):
        dirname = currdir + 'import_data' + os.sep + 'a' + os.sep
        context = {}
        m = pyutilib.misc.import_file(dirname + 'tfile.py', context=context)
        self.assertEqual(id(m), id(context['tfile']))
        self.assertEqual(m.f(), 'a')
        dirname = currdir + 'import_data' + os.sep + 'b' + os.sep
        m = pyutilib.misc.import_file(
            dirname + 'tfile.py', context=context, name='junk')
        self.assertEqual(id(m), id(context['junk']))
        self.assertEqual(m.f(), 'b')

    def test8a(self):
        dirname = currdir + 'import_data' + os.sep + 'a' + os.sep
        context = {}
        m = pyutilib.misc.import_file(dirname + 'tfile.py', context=context)
        self.assertEqual(id(m), id(context['tfile']))
        self.assertEqual(m.f(), 'a')
        dirname = currdir + 'import_data' + os.sep + 'b' + os.sep
        m = pyutilib.misc.import_file(dirname + 'tfile.py', context=context)
        self.assertEqual(id(m), id(context['tfile']))
        self.assertEqual(m.f(), 'a')

    def test9(self):
        try:
            pyutilib.misc.import_file('foo/bar')
        except ImportError:
            pass
        else:
            self.fail('Module does not exist. Expected ImportError.')

    def test9a(self):
        try:
            pyutilib.misc.import_file('foo/bar', name='junk')
        except ImportError:
            pass
        else:
            self.fail('Module does not exist. Expected ImportError.')

    def test10(self):
        try:
            pyutilib.misc.import_file('foo.bar')
        except ImportError:
            pass
        else:
            self.fail('Module does not exist. Expected ImportError.')

    def test10a(self):
        try:
            pyutilib.misc.import_file('foo.bar', name='junk')
        except ImportError:
            pass
        else:
            self.fail('Module does not exist. Expected ImportError.')

    def test11(self):
        try:
            pyutilib.misc.import_file('baz/foo.bar')
        except ImportError:
            pass
        else:
            self.fail('Module does not exist. Expected ImportError.')

    def test11a(self):
        try:
            pyutilib.misc.import_file('baz/foo.bar', name='junk')
        except ImportError:
            pass
        else:
            self.fail('Module does not exist. Expected ImportError.')

    def test12(self):
        dirname = currdir + 'import_data'
        sys.path.insert(0, dirname)
        context = {}
        m = pyutilib.misc.import_file('tfile1.0', context=context)
        self.assertEqual(id(m), id(context['tfile1.0']))
        self.assertEqual(m.f(), 'tfile1.0')
        sys.path.remove(dirname)

    def test12a(self):
        dirname = currdir + 'import_data'
        sys.path.insert(0, dirname)
        context = {}
        m = pyutilib.misc.import_file('tfile1.0', context=context, name='junk')
        self.assertEqual(id(m), id(context['junk']))
        self.assertEqual(m.f(), 'tfile1.0')
        sys.path.remove(dirname)

    def test13(self):
        dirname = currdir + 'import_data' + os.sep
        context = {}
        m = pyutilib.misc.import_file(dirname + 'tfile1.0', context=context)
        self.assertEqual(id(m), id(context['tfile1.0']))
        self.assertEqual(m.f(), 'tfile1.0')

    def test13a(self):
        dirname = currdir + 'import_data' + os.sep
        context = {}
        m = pyutilib.misc.import_file(
            dirname + 'tfile1.0', context=context, name='junk')
        self.assertEqual(id(m), id(context['junk']))
        self.assertEqual(m.f(), 'tfile1.0')

    def test14(self):
        context = {}
        m = pyutilib.misc.import_file(
            'pyutilib.misc.tests.import_data.a.tfile', context=context)
        self.assertEqual(
            id(m), id(context['pyutilib.misc.tests.import_data.a.tfile']))
        self.assertEqual(m.f(), 'a')

# Apply decorator explicitly
TestRunFile = unittest.skipIf(not _runpy, "Cannot import 'runpy'")(TestRunFile)

if __name__ == "__main__":
    unittest.main()



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

__all__ = ('get_nameserver', 'get_dispatchers', 'shutdown_pyro_components')

import os
import sys
import time
import random
import socket

# For now, default to using Pyro3 if available
# otherwise, check for Pyro4
Pyro = None
using_pyro3 = False
using_pyro4 = False
try:
    import Pyro
    import Pyro.core
    import Pyro.naming
    using_pyro3 = True
    using_pyro4 = False
except:
    try:
        import Pyro4
        import Pyro4.naming
        #Pyro4.config.SERIALIZERS_ACCEPTED.add('pickle')
        #Pyro4.config.SERIALIZER = 'pickle'
        Pyro = Pyro4
        using_pyro3 = False
        using_pyro4 = True
    except:
        Pyro = None
        using_pyro3 = False
        using_pyro4 = False

_pyro = Pyro

if sys.version_info >= (3, 0):
    xrange = range
    import queue as Queue
else:
    import Queue

_connection_problem = None
if using_pyro3:
    _connection_problem = _pyro.errors.ProtocolError
elif using_pyro4:
    _connection_problem = _pyro.errors.TimeoutError


def get_nameserver(host=None, port=None, num_retries=30, caller_name="Unknown"):

    if _pyro is None:
        raise ImportError("Pyro or Pyro4 is not available")

    timeout_upper_bound = 5.0

    if not host is None:
        os.environ['PYRO_NS_HOSTNAME'] = host
    elif 'PYRO_NS_HOSTNAME' in os.environ:
        host = os.environ['PYRO_NS_HOSTNAME']

    # Deprecated in Pyro3
    # Removed in Pyro4
    if using_pyro3:
        _pyro.core.initServer()

    ns = None

    for i in xrange(0, num_retries + 1):
        try:
            if using_pyro3:
                ns = _pyro.naming.NameServerLocator().getNS(
                    host=host, port=port)
            else:
                ns = _pyro.locateNS(host=host, port=port)
            break
        except _pyro.errors.NamingError:
            pass
        except _connection_problem:
            # this can occur if the server is too busy.
            pass

        # we originally had a single sleep timeout value, hardcoded to 1 second.
        # the problem with this approach is that if a large number of concurrent
        # processes fail, then they will all re-attempt at roughly the same
        # time. causing more contention than is necessary / desirable. by randomizing
        # the sleep interval, we are hoping to distribute the number of clients
        # attempting to connect to the name server at any given time.
        # TBD: we should eventually read the timeout upper bound from an enviornment
        #      variable - to support cases with a very large (hundreds to thousands)
        #      number of clients.
        if i < num_retries:
            sleep_interval = random.uniform(1.0, timeout_upper_bound)
            print("%s failed to locate name server after %d attempts - "
                  "trying again in %5.2f seconds." %
                  (caller_name, i + 1, sleep_interval))
            time.sleep(sleep_interval)

    if ns is None:
        print("%s could not locate nameserver (attempts=%d)" %
              (caller_name, num_retries + 1))
        raise SystemExit

    return ns


def get_dispatchers(group=":PyUtilibServer",
                    host=None,
                    port=None,
                    num_dispatcher_tries=30,
                    min_dispatchers=1,
                    caller_name=None,
                    ns=None):

    if ns is None:
        ns = get_nameserver(host=host, port=port, caller_name=caller_name)
    else:
        assert caller_name is None
        assert host is None
        assert port is None

    if ns is None:
        raise RuntimeError("Failed to locate Pyro name "
                           "server on the network!")

    cumulative_sleep_time = 0.0
    dispatchers = []
    for i in xrange(0, num_dispatcher_tries):
        ns_entries = None
        if using_pyro3:
            for (name, uri) in ns.flatlist():
                if name.startswith(":PyUtilibServer.dispatcher."):
                    if (name, uri) not in dispatchers:
                        dispatchers.append((name, uri))
        elif using_pyro4:
            for name in ns.list(prefix=":PyUtilibServer.dispatcher."):
                uri = ns.lookup(name)
                if (name, uri) not in dispatchers:
                    dispatchers.append((name, uri))
        if len(dispatchers) >= min_dispatchers:
            break
    return dispatchers

#
# a utility for shutting down Pyro-related components, which at the
# moment is restricted to the name server and any dispatchers. the
# mip servers will come down once their dispatcher is shut down.
# NOTE: this is a utility that should eventually become part of
#       pyutilib.pyro, but because is prototype, I'm keeping it
#       here for now.
#


def shutdown_pyro_components(host=None,
                             port=None,
                             num_retries=30,
                             ns=None,
                             caller_name="Unknown"):

    if _pyro is None:
        raise ImportError("Pyro or Pyro4 is not available")

    if ns is None:
        ns = get_nameserver(host=host,
                            port=port,
                            num_retries=num_retries,
                            caller_name=caller_name)
    if ns is None:
        print("***WARNING - Could not locate name server "
              "- Pyro components will not be shut down")
        return

    if using_pyro3:
        ns_entries = ns.flatlist()
        for (name, uri) in ns_entries:
            if name.startswith(":PyUtilibServer.dispatcher."):
                try:
                    ns.unregister(name)
                    proxy = _pyro.core.getProxyForURI(uri)
                    proxy.shutdown()
                except:
                    pass
        for (name, uri) in ns_entries:
            if name == ":Pyro.NameServer":
                try:
                    proxy = _pyro.core.getProxyForURI(uri)
                    proxy._shutdown()
                    proxy._release()
                except:
                    pass
    elif using_pyro4:
        for name in ns.list(prefix=":PyUtilibServer.dispatcher."):
            try:
                uri = ns.lookup(name)
                ns.remove(name)
                proxy = _pyro.Proxy(uri)
                proxy.shutdown()
                proxy._pyroRelease()
            except:
                pass
        print("")
        print("*** NameServer must be shutdown manually when using Pyro4 ***")
        print("")


def set_maxconnections(max_allowed_connections=None):

    #
    # **NOTE: For some reason with Pyro3 we need to add 1 to this
    #         option in order to to get behavior that makes sense
    #         and matches behavior with Pyro4. For instance,
    #         running a dispatcher with a single client and a single
    #         server requires PYRO_MAXCONNECTIONS=3 with Pyro3 and
    #         requires THREADPOOL_SIZE=2 with Pyro4.
    #
    if max_allowed_connections is None:
        max_pyro_connections_envname = "PYUTILIB_PYRO_MAXCONNECTIONS"
        if max_pyro_connections_envname in os.environ:
            new_val = int(os.environ[max_pyro_connections_envname])
            print("Overriding %s default for maximum number of proxy "
                  "connections to %s, based on specification provided by "
                  "%s environment variable." %
                  ("Pyro" if using_pyro3 else "Pyro4", new_val,
                   max_pyro_connections_envname))
            if using_pyro3:
                _pyro.config.PYRO_MAXCONNECTIONS = new_val + 1
            else:
                _pyro.config.THREADPOOL_SIZE = new_val
    else:
        print("Overriding %s default for maximum number of proxy "
              "connections to %s, based on specification provided by "
              "max_allowed_connections keyword" %
              ("Pyro" if using_pyro3 else "Pyro4", max_allowed_connections))
        if using_pyro3:
            _pyro.config.PYRO_MAXCONNECTIONS = max_allowed_connections + 1
        else:
            _pyro.config.THREADPOOL_SIZE = max_allowed_connections


def bind_port(sock, host="127.0.0.1"):
    """Bind the socket to a free port and return the port number.
    Relies on ephemeral ports in order to ensure we are using an
    unbound port.  This is important as many tests may be running
    simultaneously, especially in a buildbot environment.  This method
    raises an exception if the sock.family is AF_INET and sock.type is
    SOCK_STREAM, *and* the socket has SO_REUSEADDR or SO_REUSEPORT set
    on it.  Tests should *never* set these socket options for TCP/IP
    sockets.  The only case for setting these options is testing
    multicasting via multiple UDP sockets.

    Additionally, if the SO_EXCLUSIVEADDRUSE socket option is
    available (i.e.  on Windows), it will be set on the socket.  This
    will prevent anyone else from bind()'ing to our host/port for the
    duration of the test.

    This code is copied from the stdlib's test.test_support module.
    """
    if sock.family in (socket.AF_INET, socket.AF_INET6
                      ) and sock.type == socket.SOCK_STREAM:
        if hasattr(socket, "SO_EXCLUSIVEADDRUSE"):
            sock.setsockopt(socket.SOL_SOCKET, socket.SO_EXCLUSIVEADDRUSE, 1)
    if sock.family == socket.AF_INET:
        if host == 'localhost':
            sock.bind(('127.0.0.1', 0))
        else:
            sock.bind((host, 0))
    elif sock.family == socket.AF_INET6:
        if host == 'localhost':
            sock.bind(('::1', 0, 0, 0))
        else:
            sock.bind((host, 0, 0, 0))
    else:
        raise CommunicationError("unsupported socket family: " + sock.family)
    return sock.getsockname()[1]


"""
    if sock.family == socket.AF_INET and sock.type == socket.SOCK_STREAM:
        if hasattr(socket, 'SO_REUSEADDR'):
            if sock.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR) == 1:
                raise TestFailed("tests should never set the SO_REUSEADDR "
                                 "socket option on TCP/IP sockets!")
        if hasattr(socket, 'SO_REUSEPORT'):
            try:
                if sock.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT) == 1:
                    raise TestFailed("tests should never set the SO_REUSEPORT "
                                     "socket option on TCP/IP sockets!")
            except OSError:
                # Python's socket module was compiled using modern
                # headers thus defining SO_REUSEPORT but this process
                # is running under an older kernel that does not
                # support SO_REUSEPORT.
                pass
        if hasattr(socket, 'SO_EXCLUSIVEADDRUSE'):
            sock.setsockopt(socket.SOL_SOCKET, socket.SO_EXCLUSIVEADDRUSE, 1)

    sock.bind((host, 0))
    port = sock.getsockname()[1]
    return port
"""


def find_unused_port(family=socket.AF_INET, socktype=socket.SOCK_STREAM):
    """Returns an unused port that should be suitable for binding.
    This is achieved by creating a temporary socket with the same
    family and type as the 'sock' parameter (default is AF_INET,
    SOCK_STREAM), and binding it to the specified host address
    (defaults to 0.0.0.0) with the port set to 0, eliciting an unused
    ephemeral port from the OS.  The temporary socket is then closed
    and deleted, and the ephemeral port is returned.

    This code is copied from the stdlib's test.test_support module.
    """

    tempsock = socket.socket(family, socktype)
    port = bind_port(tempsock)
    tempsock.close()
    del tempsock
    return port



import os
import re
import six
import pyutilib.misc
from pyutilib.dev.runtests import runPyUtilibTests

import pyutilib.th as unittest

class Test_Runtests(unittest.TestCase):
    def _run(self, args):
        oldCat = os.environ.get('PYUTILIB_UNITTEST_CATEGORY',None)
        if oldCat is not None:
            del os.environ['PYUTILIB_UNITTEST_CATEGORY']
        stream_out = six.StringIO()
        pyutilib.misc.setup_redirect(stream_out)
        runPyUtilibTests(['nosetests', '-v', '--no-xunit'] + args + 
                         ['pyutilib.th.tests.test_pyunit'],
                         use_exec=False)
        pyutilib.misc.reset_redirect()
        if oldCat is not None:
            os.environ['PYUTILIB_UNITTEST_CATEGORY'] = oldCat

        result = []
        for line in stream_out.getvalue().splitlines():
            if '...' not in line:
                continue
            g = re.match('(\S+) \(([^\)]+)\)', line)
            if g:
                result.append("%s.%s" % (g.group(2), g.group(1)))
        return result

    def test_all(self):
        result = self._run(['--cat=all'])
        ref = [
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_custom',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_expensive',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile_smoke',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_multi',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_noCategory',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_notExpensive',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_smoke',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_custom',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_expensive',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile_smoke',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_multi',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_noCategory',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_notExpensive',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_smoke',
        ]
        self.assertEqual(sorted(result), sorted(ref))

    def test_noCategory(self):
        result = self._run([])
        ref = [
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_expensive',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile_smoke',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_multi',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_notExpensive',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_smoke',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_expensive',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile_smoke',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_multi',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_notExpensive',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_smoke',
        ]
        self.assertEqual(sorted(result), sorted(ref))


    def test_smoke(self):
        result = self._run(['--cat=smoke'])
        ref = [
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_expensive',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile_smoke',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_multi',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_notExpensive',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_smoke',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_expensive',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile_smoke',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_multi',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_notExpensive',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_smoke',
        ]
        self.assertEqual(sorted(result), sorted(ref))


    def test_custom(self):
        result = self._run(['--cat=custom'])
        ref = [
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_expensive',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile_smoke',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_multi',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_notExpensive',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_smoke',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_expensive',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile_smoke',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_multi',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_notExpensive',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_smoke',
        ]
        self.assertEqual(sorted(result), sorted(ref))


    def test_expensive(self):
        result = self._run(['--cat=expensive'])
        ref = [
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_custom',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_expensive',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile_smoke',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_multi',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_notExpensive',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_smoke',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_custom',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_expensive',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile_smoke',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_multi',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_notExpensive',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_smoke',
        ]
        self.assertEqual(sorted(result), sorted(ref))

    def test_expensive_AND_smoke(self):
        result = self._run(['--cat=expensive,smoke'])
        ref = [
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_expensive',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile_smoke',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_multi',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_notExpensive',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_smoke',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_expensive',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile_smoke',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_multi',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_notExpensive',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_smoke',
        ]
        self.assertEqual(sorted(result), sorted(ref))

    def test_expensive_OR_smoke(self):
        result = self._run(['--cat=expensive', '--cat=smoke'])
        ref = [
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_custom',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_expensive',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile_smoke',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_multi',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_notExpensive',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_smoke',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_custom',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_expensive',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile_smoke',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_multi',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_notExpensive',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_smoke',
        ]
        self.assertEqual(sorted(result), sorted(ref))

    def test_NOT_expensive(self):
        result = self._run(['--cat=!expensive'])
        ref = [
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_expensive',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile_smoke',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_multi',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_noCategory',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_notExpensive',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_smoke',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_expensive',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile_smoke',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_multi',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_noCategory',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_notExpensive',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_smoke',
        ]
        self.assertEqual(sorted(result), sorted(ref))

    def test_fragile(self):
        result = self._run(['--cat=fragile'])
        ref = [
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_expensive',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile_smoke',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_multi',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_notExpensive',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_smoke',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_expensive',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile_smoke',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_multi',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_notExpensive',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_smoke',
        ]
        self.assertEqual(sorted(result), sorted(ref))


    def test_fragile_AND_smoke(self):
        result = self._run(['--cat=fragile,smoke'])
        ref = [
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_expensive',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile_smoke',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_multi',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_notExpensive',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_smoke',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_expensive',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile_smoke',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_multi',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_notExpensive',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_smoke',
        ]
        self.assertEqual(sorted(result), sorted(ref))


    def test_fragile_OR_smoke(self):
        result = self._run(['--cat=fragile', '--cat=smoke'])
        ref = [
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_expensive',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_fragile_smoke',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_multi',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestExpensive.test_notExpensive',
            'pyutilib.th.tests.test_pyunit.TestExpensive.test_smoke',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_custom',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_expensive',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_fragile_smoke',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_multi',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_noCategory',
            #'pyutilib.th.tests.test_pyunit.TestNoCategory.test_notExpensive',
            'pyutilib.th.tests.test_pyunit.TestNoCategory.test_smoke',
        ]
        self.assertEqual(sorted(result), sorted(ref))





#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________
#
# Class to encapsulate a progress indicator

__all__ = ['progress']

import sys
import time
import unittest


class progressException(Exception):
    'Error to raise for any recursive problem.'


class progress:

    def __init__(self, period=1):
        self._twissler = ["|", "/", "-", "\\", "|"]
        self._state = 0
        self._period = period
        self._ctr = period

    def getStart(self):
        sys.stdout.write('\t[  ')  # include 2 spaces for the twissler
        sys.stdout.flush()

    def getStart(self, text):
        sys.stdout.write('\t %s [  ' %
                         (text))  # include 2 spaces for the twissler
        sys.stdout.flush()

    def moveOn(self):
        try:
            self._ctr += 1
            if self._ctr <= self._period:
                return
            self._ctr = 1
            self._state += 1
            if self._state >= 5:
                self._state = 1
            sys.stdout.write(
                chr(8) + chr(8) + self._twissler[self._state] + ']')
            sys.stdout.flush()
        except:
            raise progressException('failed to progress')


class TestCase(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testProgress(self):
        p = progress()
        p.getStart("HERE")
        for a in range(0, 10, 1):
            p.moveOn()
            time.sleep(0.2)


if __name__ == '__main__':
    widgetTestSuite = unittest.TestSuite()
    widgetTestSuite.addTest(TestCase("testProgress"))
    runner = unittest.TextTestRunner()
    runner.run(widgetTestSuite)



#  _________________________________________________________________________
#
#  PyUtilib: A Python utility library.
#  Copyright (c) 2008 Sandia Corporation.
#  This software is distributed under the BSD License.
#  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
#  the U.S. Government retains certain rights in this software.
#  _________________________________________________________________________

__all__ = ['unique_id', 'reset_id_counter']


def unique_id():
    unique_id.counter += 1
    return unique_id.counter


unique_id.counter = 0


def reset_id_counter():
    global unique_id
    unique_id.counter = 0



#
# Copyright (C) 2009, All Rights Reserved
# David Beazley
# Permission granted to distribute with PyUtilib under the BSD license
#


def coroutine(func):

    def start(*args, **kwargs):
        cr = func(*args, **kwargs)
        #print 'y',cr.send(None)
        return cr

    return start
